#!/usr/bin/env python

# main script for full Ctrax tracking tests
# JAB 5/10/11

import os
from subprocess import call
import tempfile
import time
from types import NoneType, BooleanType, IntType, StringType, ListType, TupleType

from numpy import array

import datacompare
import filelist
from Ctrax import annfiles, movies

TEST_COMMAND_LINE = False
TEST_ACCURACY = True


def exec_ctrax( params, filename, to_null ):
    cmd = "Ctrax --Interactive=False --Input=%s " % filename
    cmd += params
    if to_null:
        cmd += " > /dev/null"

    print time.asctime(), cmd
    status = call( cmd, shell=True )
    if status == 139:
        print "**caught seg-fault on exit"
    elif status != 0:
        exit( status )


def check_expected_count( load_file, n_expected_frames ):
    n_expected_frames = max( n_expected_frames, 0 )
    ann_file = annfiles.AnnotationFile( load_file, readonly=True )
    if len( ann_file ) != n_expected_frames:
        print "*************** expected %d frames in output file but Ctrax wrote %d **************" % (n_expected_frames, len( ann_file ))


def get_val_list( val, parms, filename ):    
    if type( val ) is ListType or type( val ) is TupleType:
        return val
    elif type( val ) is NoneType:
        return (True, False)
    elif type( val ) is IntType:
        return [val]
    elif type( val ) is StringType:
        basename, ext = os.path.splitext( filename )
        tempdata = tempfile.mkstemp( suffix=val )
        os.close( tempdata[0] )
        os.remove( tempdata[1] )
        return [tempdata[1]]
    else:
        raise NotImplementedError
        

def main():
    ## test command-line execution ##
    if TEST_COMMAND_LINE:

        # get filename; one short movie
        files = filelist.CtraxFileList( ['short'] )
        tmp_files = files.movie_tempfiles()
        filename = tmp_files[0]
        settings_filename = files.ann_files()[0]

        # read number of frames in movie (to test offset parameters)
        mov = movies.Movie( filename )
        n_possible_frames = mov.get_n_frames()

        # run each combination of command-line parameters
        parms = {'Output': '.ann',
                 'AutoEstimateBackground': None, # => True/False
                 'AutoEstimateShape': None,
                 'CompressMovie': '.sbfmf',
                 'Matfile': '.mat',
                 'DiagnosticsFile': '_ctraxdiagnostics.txt',
                 'FirstFrameTrack': [10],
                 'LastFrameTrack': [90]}

        cl_runtimes = []
        cl_framestracked = []
        for np in range( 2, len( parms ) + 1 ): # np = number of parameters in this run
            func_str = '\n'
            p0 = -1
            for p in range( np ):
                func_str += 3*2*p*' ' + "for p%d in range( p%d + 1, len( parms ) ):\n" % (p + 1, p)
                
                func_str += 3*(2*p + 1)*' '  + "key%d = parms.keys()[p%d]\n" % (p + 1, p + 1)
                func_str += 3*(2*p + 1)*' '  + "val%d = parms[key%d]\n" % (p + 1, p + 1)

                # get correct values to use for this key
                func_str += 3*(2*p + 1)*' ' + "val%d_list = get_val_list( val%d, parms, filename )\n\n" % (p + 1, p + 1)

                # set to run with each value
                func_str += 3*(2*p + 1)*' '  + "for use_val%d in val%d_list:\n\n" % (p + 1, p + 1)

            # calculate expected number of tracked frames
            func_str += 3*2*np*' ' + 'n_expected_frames = n_possible_frames\n'
            for p in range( np ):
                func_str += 3*2*np*' '
                if p > 0:
                    func_str += 'el'
                func_str += 'if key%d.lower() == "lastframetrack":\n' % (p + 1)
                func_str += 3*(2*np + 1)*' ' + 'n_expected_frames = min( n_expected_frames, use_val%d )\n' % (p + 1)
            for p in range( np ):
                func_str += 3*2*np*' '
                if p > 0:
                    func_str += 'el'
                func_str += 'if key%d.lower() == "firstframetrack":\n' % (p + 1)
                func_str += 3*(2*np + 1)*' ' + 'n_expected_frames -= use_val%d\n' % (p + 1)

            # make command-line string for Ctrax
            func_str += "\n"
            func_str += 3*2*np*' ' + 'cmd = \"--SettingsFile=%s ' % (settings_filename)
            for p in range( np ):
                func_str += '--%s=%s '
            func_str += '" % ('
            for p in range( np ):
                func_str += 'key%d, use_val%d, ' % (p + 1, p + 1)
            func_str += ')\n'

            func_str += 3*2*np*' ' + 'starttime = time.time()\n'

            # run
            func_str += 3*2*np*' ' + 'exec_ctrax( cmd, filename, True )\n\n'

            # calculate statistics, if # frames is large enough to count
            func_str += 3*2*np*' ' + 'if n_expected_frames > 15:\n'
            func_str += 3*(2*np + 1)*' ' + 'cl_runtimes.append( time.time() - starttime )\n'
            func_str += 3*(2*np + 1)*' ' + 'cl_framestracked.append( n_expected_frames )\n\n'

            # make sure each output file got created correctly
            func_str += 3*2*np*' ' + 'files_to_rm = []\n\n'
            for p in range( np ):
                func_str += 3*2*np*' ' + "if type( use_val%d ) is StringType:\n" % (p + 1)
                func_str += 3*(2*np + 1)*' ' + "if os.path.isfile( use_val%d ):\n" % (p + 1)
                func_str += 3*(2*np + 2)*' ' + "files_to_rm.append( use_val%d )\n" % (p + 1)
                func_str += 3*(2*np + 1)*' ' + "else:\n"
                func_str += 3*(2*np + 2)*' ' + 'print "*************** Ctrax didn\'t create file for parameter %%s **************" %% key%d\n' % (p + 1)

                # make sure that the correct number of frames were output
                func_str += 3*2*np*' ' + 'elif key%d.lower() == "firstframetrack" or key%d.lower() == "lastframetrack":\n' % (p + 1, p + 1)
                func_str += 3*(2*np + 1)*' ' + "load_file = '%s'\n" % (filename + '.ann')
                for q in range( np ):
                    if q == p: continue
                    func_str += 3*(2*np + 1)*' '
                    if (p == 0 and q > 1) or (p != 0 and q > 0):
                        func_str += 'el'
                    func_str += 'if key%d.lower() == "output":\n' % (q + 1)
                    func_str += 3*(2*np + 2)*' ' + "load_file = use_val%d\n" % (q + 1)
                func_str += 3*(2*np + 1)*' ' + "check_expected_count( load_file, n_expected_frames )\n\n"

            # delete output files
            func_str += 3*2*np*' ' + "for rmfile in files_to_rm:\n"
            func_str += 3*(2*np + 1)*' ' + 'os.remove( rmfile )\n\n'

            #for p in range( 2*np, 0, -1 ):
            #    func_str += 3*p*' ' + 'break\n'
            #print func_str

            exec( func_str )

        # test ResumeTracking
        basename, ext = os.path.splitext( filename )
        tempdata = tempfile.mkstemp( suffix='.ann' )
        os.close( tempdata[0] )
        os.remove( tempdata[1] )
        outfile = tempdata[1]

        exec_ctrax( "--SettingsFile=%s --Output=%s --LastFrameTrack=20" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, 20 )
        exec_ctrax( "--SettingsFile=%s --Output=%s --ResumeTracking=True" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames )
        os.remove( outfile )

        exec_ctrax( "--SettingsFile=%s --Output=%s --FirstFrameTrack=10 --LastFrameTrack=20" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, 10 )
        exec_ctrax( "--SettingsFile=%s --Output=%s --ResumeTracking=True" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames - 10 )
        os.remove( outfile )

        exec_ctrax( "--SettingsFile=%s --Output=%s --FirstFrameTrack=90" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames - 90 )
        exec_ctrax( "--SettingsFile=%s --Output=%s --ResumeTracking=True" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames - 90 )
        os.remove( outfile )

        exec_ctrax( "--SettingsFile=%s --Output=%s" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames )
        exec_ctrax( "--SettingsFile=%s --Output=%s --ResumeTracking=True" % (settings_filename, outfile), filename, True )
        check_expected_count( outfile, n_possible_frames )
        os.remove( outfile )


    ## test for tracking accuracy ##
    if TEST_ACCURACY:

        # get filenames
        files = filelist.CtraxFileList()
        tmp_files = files.movie_tempfiles()

        # run Ctrax on each file
        runtimes = []
        for filename in tmp_files:
            starttime = time.time()

            exec_ctrax( "", filename, True )

            runtimes.append( time.time() - starttime )

        # get Matlab to make readable versions of the test suite's original (fixed) MAT-files
        call( "matlab -nodisplay -nojvm -nosplash -r 'make_test_data; exit'", shell=True )

        # compare original MAT-data with newly generated MAT-data
        comparator = datacompare.CtraxDataComparator( files.resaved_mat_files(),
                                                      files.resaved_mat_tempfiles() )
        out_filename = comparator.save_data( files, runtimes )

        # run visualization in Matlab
        print "wrote tracking data to", out_filename
        call( "matlab -r \"plot_test_data_comparisons( '" + out_filename + "' )\"", shell=True )


    if TEST_COMMAND_LINE:
        total_framestracked = array( cl_framestracked ).sum()
        total_runtime = array( cl_runtimes ).sum()
        tracking_speed = (array( cl_framestracked )/array( cl_runtimes )).mean()
        print "mean speed for command-line tests: %f fps (%d frames in %d sec)" % (tracking_speed, total_framestracked, total_runtime)
    

if __name__ == '__main__':
    main()
