#!/bin/bash

# upload script for Ctrax PPA
# JAB 2/14/12

# Ubuntu must have extra packages installed:
# ubuntu-dev-tools
# (maybe also dput, if not included)

# Launchpad must have a GPG key and an SSH key for user.
# See their instructions.
# https://launchpad.net/~ctrax/+editpgpkeys
# https://launchpad.net/~ctrax/+editsshkeys
# The GPG key's passphrase might be needed up to 4 times
# during build/sign/upload.

keyname='E56DDCF5' # JAB Precise 64
#keyname='D9E19AA4' # JAB Lucid 32

##### these lines go in ~/.dput.cf to configure dput #####
#[ctrax]
#fqdn = ppa.launchpad.net
#method = sftp
#incoming = ~ctrax/ppa/ubuntu/
#login = ctrax
#allow_unsigned_uploads = 0
##########################


##### This sequence of commands builds and uploads to Launchpad. #####
# Building for a second Ubuntu series requires performing each step
# manually, with some additions as noted in the comments ("MANUAL").

# requires stdeb package, from PyPI (same command as makedeb.sh)
python setup.py --command-packages=stdeb.command bdist_deb

# get $version from version.txt
version="`cat version.txt`"
cd deb_dist/ctrax-$version/

# build and sign .deb files
# get key ID (-k) from the output of  gpg --list-keys
# (must be the same key that Launchpad has registered)
debuild -S -sa -k$keyname

# change version name "unstable" to an Ubuntu version in changelog
# MANUAL: also change version number -1 to -2
perl -pi -e s/unstable/oneiric/ debian/changelog
# build again (first build created debian/changelog from setup.py config)
debuild -S -sa -k$keyname

# MANUAL:
# download the ctrax_$version.orig.tar.gz file from the version already built
#   into the deb_dist/ directory
# regenerate the signed changes file with "debuild -S -k$keyname"

cd ..
# MANUAL: change the -1 to -2 here
dput ctrax ctrax_$version-1_source.changes

